@@ -1,3 +1,27 @@
+/*
+    Authors:
+        John Dennis <jdennis.redhat.com>
+        Esmond Pitt <ejp@ausmelb.oz.AU>
+
+    This implementation was based on a 3/7/1989 public domain submission
+    to comp.sources.misc by Esmond Pitt <ejp@ausmelb.oz.AU>.
+
+    Copyright (C) 2009 Red Hat
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU Lesser General Public License as published by
+    the Free Software Foundation; either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
 /*****************************************************************************/
 /******************************** Documentation ******************************/
 /*****************************************************************************/
@@ -34,6 +58,14 @@
   * concatenation property
   */
 
+#define halloc(table, size) table->halloc(size, table->halloc_pvt)
+#define hfree(table, ptr) table->hfree(ptr, table->halloc_pvt)
+#define hdelete_callback(table, type, entry) do { \
+    if (table->delete_callback) { \
+        table->delete_callback(entry, type, table->delete_pvt); \
+    } \
+} while(0)
+
 /*****************************************************************************/
 /************************** Internal Type Definitions ************************/
 /*****************************************************************************/
@@ -56,9 +88,11 @@ struct hash_table_str {
     unsigned int    directory_size_shift;
     unsigned long   segment_size;
     unsigned int    segment_size_shift;
-    hash_delete_callback delete_callback;
-    hash_alloc_func alloc;
-    hash_free_func free;
+    hash_delete_callback *delete_callback;
+    void *delete_pvt;
+    hash_alloc_func *halloc;
+    hash_free_func *hfree;
+    void *halloc_pvt;
     segment_t **directory;
 #ifdef HASH_STATISTICS
     hash_statistics_t statistics;
@@ -117,6 +151,16 @@ int debug_level = 1;
 /***************************  Internal Functions  ****************************/
 /*****************************************************************************/
 
+static void *sys_malloc_wrapper(size_t size, void *pvt)
+{
+    return malloc(size);
+}
+
+static void sys_free_wrapper(void *ptr, void *pvt)
+{
+    return free(ptr);
+}
+
 static address_t convert_key(hash_key_t *key)
 {
     address_t h;
@@ -224,7 +268,8 @@ static int expand_table(hash_table_t *table)
         new_segment_dir = new_address >> table->segment_size_shift;
         new_segment_index = new_address & (table->segment_size-1); /* new_address % segment_size */
         if (new_segment_index == 0) {
-            if ((table->directory[new_segment_dir] = (segment_t *) table->alloc(table->segment_size * sizeof(segment_t))) == NULL) {
+            table->directory[new_segment_dir] = (segment_t *)halloc(table, table->segment_size * sizeof(segment_t));
+            if (table->directory[new_segment_dir] == NULL) {
                 return HASH_ERROR_NO_MEMORY;
             }
             memset(table->directory[new_segment_dir], 0, table->segment_size * sizeof(segment_t));
@@ -346,7 +391,7 @@ static int contract_table(hash_table_t *table)
          */
         if (old_segment_index == 0) {
             table->segment_count--;
-            table->free(table->directory[old_segment_dir]);
+            hfree(table, table->directory[old_segment_dir]);
         }
 
 #ifdef DEBUG
@@ -441,25 +486,34 @@ const char* hash_error_string(int error)
 }
 
 
-int hash_create(unsigned long count, hash_table_t **tbl, hash_delete_callback delete_callback)
+int hash_create(unsigned long count, hash_table_t **tbl,
+                hash_delete_callback *delete_callback,
+                void *delete_private_data)
 {
-    return hash_create_ex(count, tbl, 0, 0, 0, 0, NULL, NULL, delete_callback);
+    return hash_create_ex(count, tbl, 0, 0, 0, 0,
+                          NULL, NULL, NULL,
+                          delete_callback,
+                          delete_private_data);
 }
 
 int hash_create_ex(unsigned long count, hash_table_t **tbl,
-                   unsigned int directory_bits, unsigned int segment_bits,
-                   unsigned long min_load_factor, unsigned long max_load_factor,
-                   hash_alloc_func alloc_func,
-                   hash_free_func free_func,
-                   hash_delete_callback delete_callback)
+                   unsigned int directory_bits,
+                   unsigned int segment_bits,
+                   unsigned long min_load_factor,
+                   unsigned long max_load_factor,
+                   hash_alloc_func *alloc_func,
+                   hash_free_func *free_func,
+                   void *alloc_private_data,
+                   hash_delete_callback *delete_callback,
+                   void *delete_private_data)
 {
     unsigned long i;
     unsigned int n_addr_bits;
     address_t addr;
     hash_table_t *table = NULL;
 
-    if (alloc_func == NULL) alloc_func = malloc;
-    if (free_func == NULL) free_func = free;
+    if (alloc_func == NULL) alloc_func = sys_malloc_wrapper;
+    if (free_func == NULL) free_func = sys_free_wrapper;
 
     /* Compute directory and segment parameters */
     if (directory_bits == 0) directory_bits = HASH_DEFAULT_DIRECTORY_BITS;
@@ -469,12 +523,15 @@ int hash_create_ex(unsigned long count, hash_table_t **tbl,
 
     if (directory_bits + segment_bits > n_addr_bits) return EINVAL;
 
-    if ((table = (hash_table_t *) alloc_func(sizeof(hash_table_t))) == NULL) {
+    table = (hash_table_t *)alloc_func(sizeof(hash_table_t),
+                                       alloc_private_data);
+    if (table == NULL) {
         return HASH_ERROR_NO_MEMORY;
     }
     memset(table, 0, sizeof(hash_table_t));
-    table->alloc = alloc_func;
-    table->free = free_func;
+    table->halloc = alloc_func;
+    table->hfree = free_func;
+    table->halloc_pvt = alloc_private_data;
 
     table->directory_size_shift = directory_bits;
     for (i = 0, table->directory_size = 1; i < table->directory_size_shift; i++, table->directory_size <<= 1);
@@ -484,7 +541,8 @@ int hash_create_ex(unsigned long count, hash_table_t **tbl,
 
 
     /* Allocate directory */
-    if ((table->directory = (segment_t **) table->alloc(table->directory_size * sizeof(segment_t *))) == NULL) {
+    table->directory = (segment_t **)halloc(table, table->directory_size * sizeof(segment_t *));
+    if (table->directory == NULL) {
         return HASH_ERROR_NO_MEMORY;
     }
     memset(table->directory, 0, table->directory_size * sizeof(segment_t *));
@@ -502,12 +560,14 @@ int hash_create_ex(unsigned long count, hash_table_t **tbl,
     table->p = 0;
     table->entry_count = 0;
     table->delete_callback = delete_callback;
+    table->delete_pvt = delete_private_data;
 
     /*
      * Allocate initial 'i' segments of buckets
      */
     for (i = 0; i < count; i++) {
-        if ((table->directory[i] = (segment_t *) table->alloc(table->segment_size * sizeof(segment_t))) == NULL) {
+        table->directory[i] = (segment_t *)halloc(table, table->segment_size * sizeof(segment_t));
+        if (table->directory[i] == NULL) {
             hash_destroy(table);
             return HASH_ERROR_NO_MEMORY;
         }
@@ -560,17 +620,19 @@ int hash_destroy(hash_table_t *table)
                     p = s[j];
                     while (p != NULL) {
                         q = p->next;
-                        if (table->delete_callback) table->delete_callback(&p->entry);
-                        if (p->entry.key.type == HASH_KEY_STRING) table->free ((char *)p->entry.key.str);
-                        table->free((char *) p);
+                        hdelete_callback(table, HASH_TABLE_DESTROY, &p->entry);
+                        if (p->entry.key.type == HASH_KEY_STRING) {
+                            hfree(table, (char *)p->entry.key.str);
+                        }
+                        hfree(table, (char *)p);
                         p = q;
                     }
                 }
-                table->free(s);
+                hfree(table, s);
             }
         }
-        table->free(table->directory);
-        table->free(table);
+        hfree(table, table->directory);
+        hfree(table, table);
         table = NULL;
     }
     return HASH_SUCCESS;
@@ -639,7 +701,8 @@ struct hash_iter_context_t *new_hash_iter_context(hash_table_t *table)
 
     if (!table) return NULL;;
 
-    if ((iter = table->alloc(sizeof(struct _hash_iter_context_t))) == NULL) {
+    iter = halloc(table, sizeof(struct _hash_iter_context_t));
+    if (iter == NULL) {
         return NULL;
     }
 
@@ -675,7 +738,8 @@ int hash_keys(hash_table_t *table, unsigned long *count_arg, hash_key_t **keys_a
         return HASH_SUCCESS;
     }
 
-    if ((keys = table->alloc(sizeof(hash_key_t) * count)) == NULL) {
+    keys = halloc(table, sizeof(hash_key_t) * count);
+    if (keys == NULL) {
         *count_arg = -1;
         *keys_arg = NULL;
         return HASH_ERROR_NO_MEMORY;
@@ -705,7 +769,8 @@ int hash_values(hash_table_t *table, unsigned long *count_arg, hash_value_t **va
         return HASH_SUCCESS;
     }
 
-    if ((values = table->alloc(sizeof(hash_value_t) * count)) == NULL) {
+    values = halloc(table, sizeof(hash_value_t) * count);
+    if (values == NULL) {
         *count_arg = -1;
         *values_arg = NULL;
         return HASH_ERROR_NO_MEMORY;
@@ -748,7 +813,8 @@ int hash_entries(hash_table_t *table, unsigned long *count_arg, hash_entry_t **e
         return HASH_SUCCESS;
     }
 
-    if ((entries = table->alloc(sizeof(hash_entry_t) * count)) == NULL) {
+    entries = halloc(table, sizeof(hash_entry_t) * count);
+    if (entries == NULL) {
         *count_arg = -1;
         *entries_arg = NULL;
         return HASH_ERROR_NO_MEMORY;
@@ -809,7 +875,8 @@ int hash_enter(hash_table_t *table, hash_key_t *key, hash_value_t *value)
     lookup(table, key, &element, &chain);
 
     if (element == NULL) {                    /* not found */
-        if ((element = (element_t *) table->alloc(sizeof(element_t))) == NULL) {
+        element = (element_t *)halloc(table, sizeof(element_t));
+        if (element == NULL) {
             /* Allocation failed, return NULL */
             return HASH_ERROR_NO_MEMORY;
         }
@@ -823,8 +890,9 @@ int hash_enter(hash_table_t *table, hash_key_t *key, hash_value_t *value)
             break;
         case HASH_KEY_STRING:
             len = strlen(key->str)+1;
-            if ((element->entry.key.str = table->alloc(len)) == NULL) {
-                table->free(element);
+            element->entry.key.str = halloc(table, len);
+            if (element->entry.key.str == NULL) {
+                hfree(table, element);
                 return HASH_ERROR_NO_MEMORY;
             }
             memcpy((void *)element->entry.key.str, key->str, len);
@@ -902,7 +970,7 @@ int hash_delete(hash_table_t *table, hash_key_t *key)
     lookup(table, key, &element, &chain);
 
     if (element) {
-        if (table->delete_callback) table->delete_callback(&element->entry);
+        hdelete_callback(table, HASH_ENTRY_DESTROY, &element->entry);
         *chain = element->next; /* remove from chain */
         /*
          * Table too sparse?
@@ -912,8 +980,10 @@ int hash_delete(hash_table_t *table, hash_key_t *key)
                 return error;
             }
         }
-        if (element->entry.key.type == HASH_KEY_STRING) table->free ((char *)element->entry.key.str);
-        table->free(element);
+        if (element->entry.key.type == HASH_KEY_STRING) {
+            hfree(table, (char *)element->entry.key.str);
+        }
+        hfree(table, element);
         return HASH_SUCCESS;
     } else {
         return HASH_ERROR_KEY_NOT_FOUND;
