@@ -42,6 +42,9 @@
 
 #define SLASH           "/"
 
+#define EXCLUDE_EMPTY   0
+#define INCLUDE_EMPTY   1
+
 /* Name of the special collection used to store parsing errors */
 #define FILE_ERROR_SET  "ini_file_error_set"
 
@@ -236,9 +239,13 @@ static int ini_to_collection(FILE *file,
 
         switch (status) {
         case RET_PAIR:
-            /* Add line to the collection of lines */
+            /* Add line to the collection of lines.
+             * It is pretty safe in this case to just type cast the value to
+             * int32_t since it is unrealistic that ini file will ever have
+             * so many lines.
+             */
             if (lines) {
-                error = col_add_int_property(*lines, NULL, key, line);
+                error = col_add_int_property(*lines, NULL, key, (int32_t)line);
                 if (error) {
                     TRACE_ERROR_NUMBER("Failed to add line to line collection", error);
                     fclose(file);
@@ -305,8 +312,11 @@ static int ini_to_collection(FILE *file,
             if (lines) {
                 /* For easier search make line numbers for the sections negative.
                  * This would allow differentiating sections and attributes.
+                 * It is pretty safe in this case to just type cast the value to
+                 * int32_t since it is unrealistic that ini file will ever have
+                 * so many lines.
                  */
-                error = col_add_int_property(*lines, NULL, key, -1 * line);
+                error = col_add_int_property(*lines, NULL, key, (int32_t)(-1 * line));
                 if (error) {
                     TRACE_ERROR_NUMBER("Failed to add line to line collection", error);
                     fclose(file);
@@ -354,20 +364,25 @@ static int ini_to_collection(FILE *file,
             break;
 
         case RET_ERROR:
-            pe.line = line;
-            pe.error = ext_err;
-            error = col_add_binary_property(*error_list, NULL,
-                                            ERROR_TXT, &pe, sizeof(pe));
-            if (error) {
-                TRACE_ERROR_NUMBER("Failed to add error to collection", error);
-                fclose(file);
-                col_destroy_collection(current_section);
-                if (created) {
-                    col_destroy_collection(*error_list);
-                    *error_list = NULL;
+            /* Try to add to the error list only if it is present */
+            if (error_list) {
+                pe.line = line;
+                pe.error = ext_err;
+                error = col_add_binary_property(*error_list, NULL,
+                                                ERROR_TXT, &pe, sizeof(pe));
+                if (error) {
+                    TRACE_ERROR_NUMBER("Failed to add error to collection",
+                                       error);
+                    fclose(file);
+                    col_destroy_collection(current_section);
+                    if (created) {
+                        col_destroy_collection(*error_list);
+                        *error_list = NULL;
+                    }
+                    return error;
                 }
-                return error;
             }
+
             /* Exit if there was an error parsing file */
             if (error_level != INI_STOP_ON_NONE) {
                 TRACE_ERROR_STRING("Invalid format of the file", "");
@@ -379,20 +394,25 @@ static int ini_to_collection(FILE *file,
 
         case RET_INVALID:
         default:
-            pe.line = line;
-            pe.error = ext_err;
-            error = col_add_binary_property(*error_list, NULL,
-                                            WARNING_TXT, &pe, sizeof(pe));
-            if (error) {
-                TRACE_ERROR_NUMBER("Failed to add warning to collection", error);
-                fclose(file);
-                col_destroy_collection(current_section);
-                if (created) {
-                    col_destroy_collection(*error_list);
-                    *error_list = NULL;
+            /* Try to add to the error list only if it is present */
+            if (error_list) {
+                pe.line = line;
+                pe.error = ext_err;
+                error = col_add_binary_property(*error_list, NULL,
+                                                WARNING_TXT, &pe, sizeof(pe));
+                if (error) {
+                    TRACE_ERROR_NUMBER("Failed to add warning to collection",
+                                       error);
+                    fclose(file);
+                    col_destroy_collection(current_section);
+                    if (created) {
+                        col_destroy_collection(*error_list);
+                        *error_list = NULL;
+                    }
+                    return error;
                 }
-                return error;
             }
+
             /* Exit if we are told to exit on warnings */
             if (error_level == INI_STOP_ON_ANY) {
                 TRACE_ERROR_STRING("Invalid format of the file", "");
@@ -1272,13 +1292,16 @@ int get_config_item(const char *section,
     return error;
 }
 
-/* Get long value from config item */
-long get_long_config_value(struct collection_item *item,
-                           int strict, long def, int *error)
+/* Get long long value from config item */
+static long long get_llong_config_value(struct collection_item *item,
+                                        int strict,
+                                        long long def,
+                                        int *error)
 {
+    int err;
     const char *str;
     char *endptr;
-    long val = 0;
+    long long val = 0;
 
     TRACE_FLOW_STRING("get_long_config_value", "Entry");
 
@@ -1295,50 +1318,179 @@ long get_long_config_value(struct collection_item *item,
     /* Try to parse the value */
     str = (const char *)col_get_item_data(item);
     errno = 0;
-    val = strtol(str, &endptr, 10);
+    val = strtoll(str, &endptr, 10);
+    err = errno;
 
     /* Check for various possible errors */
-    if (((errno == ERANGE) &&
-        ((val == LONG_MAX) ||
-         (val == LONG_MIN))) ||
-         ((errno != 0) && (val == 0)) ||
-         (endptr == str)) {
-        TRACE_ERROR_NUMBER("Conversion failed", EIO);
+    if (err != 0) {
+        TRACE_ERROR_NUMBER("Conversion failed", err);
+        if (error) *error = err;
+        return def;
+    }
+
+    /* Other error cases */
+    if ((endptr == str) || (strict && (*endptr != '\0'))) {
+        TRACE_ERROR_NUMBER("More characters or nothing processed", EIO);
         if (error) *error = EIO;
         return def;
     }
 
-    if (strict && (*endptr != '\0')) {
-        TRACE_ERROR_NUMBER("More characters than expected", EIO);
-        if (error) *error = EIO;
-        val = def;
-    }
-
-    TRACE_FLOW_NUMBER("get_long_config_value returning", val);
+    TRACE_FLOW_NUMBER("get_long_config_value returning", (long)val);
     return val;
 }
 
+/* Get unsigned long long value from config item */
+static unsigned long long get_ullong_config_value(struct collection_item *item,
+                                                  int strict,
+                                                  unsigned long long def,
+                                                  int *error)
+{
+    int err;
+    const char *str;
+    char *endptr;
+    unsigned long long val = 0;
+
+    TRACE_FLOW_STRING("get_long_config_value", "Entry");
+
+    /* Do we have the item ? */
+    if ((item == NULL) ||
+       (col_get_item_type(item) != COL_TYPE_STRING)) {
+        TRACE_ERROR_NUMBER("Invalid argument.", EINVAL);
+        if (error) *error = EINVAL;
+        return def;
+    }
+
+    if (error) *error = EOK;
+
+    /* Try to parse the value */
+    str = (const char *)col_get_item_data(item);
+    errno = 0;
+    val = strtoull(str, &endptr, 10);
+    err = errno;
+
+    /* Check for various possible errors */
+    if (err != 0) {
+        TRACE_ERROR_NUMBER("Conversion failed", err);
+        if (error) *error = err;
+        return def;
+    }
+
+    /* Other error cases */
+    if ((endptr == str) || (strict && (*endptr != '\0'))) {
+        TRACE_ERROR_NUMBER("More characters or nothing processed", EIO);
+        if (error) *error = EIO;
+        return def;
+    }
+
+    TRACE_FLOW_NUMBER("get_long_config_value returning", (long)val);
+    return val;
+}
+
+
 /* Get integer value from config item */
 int get_int_config_value(struct collection_item *item,
-                         int strict, int def, int *error)
+                         int strict,
+                         int def,
+                         int *error)
 {
-    return get_long_config_value(item, strict, def, error);
+    long long val = 0;
+    int err = 0;
+
+    TRACE_FLOW_STRING("get_int_config_value", "Entry");
+
+    val = get_llong_config_value(item, strict, def, &err);
+    if (err == 0) {
+        if ((val > INT_MAX) || (val < INT_MIN)) {
+            val = def;
+            err = ERANGE;
+        }
+    }
+
+    if (error) *error = err;
+
+    TRACE_FLOW_NUMBER("get_int_config_value returning", (int)val);
+    return (int)val;
 }
 
 /* Get unsigned integer value from config item */
 unsigned get_unsigned_config_value(struct collection_item *item,
-                                   int strict, unsigned def, int *error)
+                                   int strict,
+                                   unsigned def,
+                                   int *error)
 {
-    return get_long_config_value(item, strict, def, error);
+    unsigned long long val = 0;
+    int err = 0;
+
+    TRACE_FLOW_STRING("get_unsigned_config_value", "Entry");
+
+    val = get_ullong_config_value(item, strict, def, &err);
+    if (err == 0) {
+        if (val > UINT_MAX) {
+            val = def;
+            err = ERANGE;
+        }
+    }
+
+    if (error) *error = err;
+
+    TRACE_FLOW_NUMBER("get_unsigned_config_value returning",
+                      (unsigned)val);
+    return (unsigned)val;
+}
+
+/* Get long value from config item */
+long get_long_config_value(struct collection_item *item,
+                           int strict,
+                           long def,
+                           int *error)
+{
+    long long val = 0;
+    int err = 0;
+
+    TRACE_FLOW_STRING("get_long_config_value", "Entry");
+
+    val = get_llong_config_value(item, strict, def, &err);
+    if (err == 0) {
+        if ((val > LONG_MAX) || (val < LONG_MIN)) {
+            val = def;
+            err = ERANGE;
+        }
+    }
+
+    if (error) *error = err;
+
+    TRACE_FLOW_NUMBER("get_long_config_value returning",
+                      (long)val);
+    return (long)val;
 }
 
 /* Get unsigned long value from config item */
 unsigned long get_ulong_config_value(struct collection_item *item,
-                                     int strict, unsigned long def, int *error)
+                                     int strict,
+                                     unsigned long def,
+                                     int *error)
 {
-    return get_long_config_value(item, strict, def, error);
+    unsigned long long val = 0;
+    int err = 0;
+
+    TRACE_FLOW_STRING("get_ulong_config_value", "Entry");
+
+    val = get_ullong_config_value(item, strict, def, &err);
+    if (err == 0) {
+        if (val > ULONG_MAX) {
+            val = def;
+            err = ERANGE;
+        }
+    }
+
+    if (error) *error = err;
+
+    TRACE_FLOW_NUMBER("get_ulong_config_value returning",
+                      (unsigned long)val);
+    return (unsigned long)val;
 }
 
+
 /* Get double value */
 double get_double_config_value(struct collection_item *item,
                                int strict, double def, int *error)
@@ -1569,13 +1721,16 @@ void free_bin_config_value(char *value)
     if (value) free(value);
 }
 
-/* Arrays of stings and integers */
-char **get_string_config_array(struct collection_item *item,
-                               const char *sep, int *size, int *error)
+/* Arrays of stings */
+static char **get_str_cfg_array(struct collection_item *item,
+                                int include,
+                                const char *sep,
+                                int *size,
+                                int *error)
 {
-    const char *defsep = ",";
     char *copy = NULL;
     char *dest = NULL;
+    char locsep[4];
     int lensep;
     char *buff;
     int count = 0;
@@ -1583,11 +1738,10 @@ char **get_string_config_array(struct collection_item *item,
     int resume_len;
     char **array;
     char *start;
-    int i, j, k;
-    int growlen = 0;
+    int i, j;
     int dlen;
 
-    TRACE_FLOW_STRING("get_string_config_array", "Entry");
+    TRACE_FLOW_STRING("get_str_cfg_array", "Entry");
 
     /* Do we have the item ? */
     if ((item == NULL) ||
@@ -1598,8 +1752,16 @@ char **get_string_config_array(struct collection_item *item,
     }
 
     /* Handle the separators */
-    if (sep == NULL) sep = defsep;
-    lensep = strnlen(sep, 3);
+    if (sep == NULL) {
+        locsep[0] = ',';
+        locsep[1] = '\0';
+        lensep = 2;
+    }
+    else {
+        strncpy(locsep, sep, 3);
+        locsep[3] = '\0';
+        lensep = strlen(locsep) + 1;
+    }
 
     /* Allocate memory for the copy of the string */
     copy = malloc(col_get_item_length(item));
@@ -1613,17 +1775,18 @@ char **get_string_config_array(struct collection_item *item,
     dest = copy;
     buff = col_get_item_data(item);
     start = buff;
-    dlen = col_get_item_length(item) - 1;
+    dlen = col_get_item_length(item);
     for(i = 0; i < dlen; i++) {
-        growlen = 1;
         for(j = 0; j < lensep; j++) {
-            if(buff[i] == sep[j]) {
+            if(buff[i] == locsep[j]) {
                 /* If we found one of the separators trim spaces around */
                 resume_len = len;
                 while (len > 0) {
                     if (isspace(start[len - 1])) len--;
                     else break;
                 }
+                TRACE_INFO_STRING("Current:", start);
+                TRACE_INFO_NUMBER("Length:", len);
                 if (len > 0) {
                     /* Save block aside */
                     memcpy(dest, start, len);
@@ -1631,52 +1794,29 @@ char **get_string_config_array(struct collection_item *item,
                     dest += len;
                     *dest = '\0';
                     dest++;
-                    len = 0;
                 }
+                else if(include) {
+                    count++;
+                    *dest = '\0';
+                    dest++;
+                }
+                if (locsep[j] == '\0') break; /* We are done */
+
                 /* Move forward and trim spaces if any */
                 start += resume_len + 1;
                 i++;
                 TRACE_INFO_STRING("Other pointer :", buff + i);
-                k = 0;
-                while(1) {
-                    TRACE_INFO_STRING("Remaining buffer :", start);
-                    while (((i + k) < dlen) && (isspace(*start))) {
-                        k++;
-                        start++;
-                    }
-                    /* May be we have another separator */
-                    TRACE_INFO_STRING("Remaining before sep check :", start);
-                    if(*start && strchr(sep, *start)) {
-                        TRACE_INFO_NUMBER("Found separator:", *start);
-                        start++;
-                        k++;
-                    }
-                    else {
-                        break;
-                    }
+                while ((i < dlen) && (isspace(*start))) {
+                    i++;
+                    start++;
                 }
-
+                len = -1; /* Len will be increased in the loop */
+                i--; /* i will be increas so we need to step back */
                 TRACE_INFO_STRING("Remaining buffer after triming spaces:", start);
-
-                if (k) i += k - 1;
-                /* Next iteration of the loop will add 1 */
-                /* Break out of the inner loop */
-                growlen = 0;
                 break;
             }
         }
-        if (growlen) len++;
-    }
-
-    TRACE_INFO_STRING("Last part :", start);
-    TRACE_INFO_NUMBER("Length :", len);
-    if(len) {
-        /* Copy the remaining piece */
-        memcpy(dest, start, len);
-        count++;
-        dest += len;
-        *dest = '\0';
-        dest++;
+        len++;
     }
 
     /* Now we know how many items are there in the list */
@@ -1702,10 +1842,27 @@ char **get_string_config_array(struct collection_item *item,
 
     if (error) *error = EOK;
     if (size) *size = count;
-    TRACE_FLOW_STRING("get_string_config_array", "Exit");
+    /* If count is 0 the copy needs to be freed */
+    if (count == 0) free(copy);
+    TRACE_FLOW_STRING("get_str_cfg_array", "Exit");
     return array;
 }
 
+/* Get array of strings from item eliminating empty tokens */
+char **get_string_config_array(struct collection_item *item,
+                               const char *sep, int *size, int *error)
+{
+    TRACE_FLOW_STRING("get_string_config_array", "Called.");
+    return get_str_cfg_array(item, EXCLUDE_EMPTY, sep, size, error);
+}
+/* Get array of strings from item preserving empty tokens */
+char **get_raw_string_config_array(struct collection_item *item,
+                                   const char *sep, int *size, int *error)
+{
+    TRACE_FLOW_STRING("get_raw_string_config_array", "Called.");
+    return get_str_cfg_array(item, INCLUDE_EMPTY, sep, size, error);
+}
+
 /* Special function to free string config array */
 void free_string_config_array(char **str_config)
 {
@@ -1719,7 +1876,11 @@ void free_string_config_array(char **str_config)
     TRACE_FLOW_STRING("free_string_config_array", "Exit");
 }
 
-/* Get an array of long values */
+/* Get an array of long values.
+ * NOTE: For now I leave just one function that returns numeric arrays.
+ * In future if we need other numeric types we can change it to do strtoll
+ * internally and wrap it for backward compatibility.
+ */
 long *get_long_config_array(struct collection_item *item, int *size, int *error)
 {
     const char *str;
@@ -1727,6 +1888,7 @@ long *get_long_config_array(struct collection_item *item, int *size, int *error)
     long val = 0;
     long *array;
     int count = 0;
+    int err;
 
     TRACE_FLOW_STRING("get_long_config_array", "Entry");
 
@@ -1750,18 +1912,25 @@ long *get_long_config_array(struct collection_item *item, int *size, int *error)
     /* Now parse the string */
     str = (const char *)col_get_item_data(item);
     while (*str) {
+
         errno = 0;
         val = strtol(str, &endptr, 10);
-        if (((errno == ERANGE) &&
-            ((val == LONG_MAX) ||
-             (val == LONG_MIN))) ||
-             ((errno != 0) && (val == 0)) ||
-             (endptr == str)) {
-            TRACE_ERROR_NUMBER("Conversion failed", EIO);
+        err = errno;
+
+        if (err) {
+            TRACE_ERROR_NUMBER("Conversion failed", err);
+            free(array);
+            if (error) *error = err;
+            return NULL;
+        }
+
+        if (endptr == str) {
+            TRACE_ERROR_NUMBER("Nothing processed", EIO);
             free(array);
             if (error) *error = EIO;
             return NULL;
         }
+
         /* Save value */
         array[count] = val;
         count++;
